package com.mcwl.communityCenter.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.constant.HttpStatus;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.core.page.TableDataInfo;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.common.utils.StringUtils;
import com.mcwl.communityCenter.constant.StatusConstant;
import com.mcwl.communityCenter.domain.CommunityAdvice;
import com.mcwl.communityCenter.domain.Invite;
import com.mcwl.communityCenter.domain.Publish;
import com.mcwl.communityCenter.domain.Question;
import com.mcwl.communityCenter.domain.dto.QuestionPageRes;
import com.mcwl.communityCenter.domain.dto.QuestionReplyDto;
import com.mcwl.communityCenter.domain.dto.QuestionRes;
import com.mcwl.communityCenter.domain.vo.QuestionVo;
import com.mcwl.communityCenter.mapper.CommunityAdviceMapper;
import com.mcwl.communityCenter.mapper.PublishMapper;
import com.mcwl.communityCenter.mapper.QuestionMapper;
import com.mcwl.communityCenter.service.InviteService;
import com.mcwl.communityCenter.service.PublishService;
import com.mcwl.communityCenter.service.QuestionService;
import com.mcwl.system.service.ISysUserService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

@Service
@RequiredArgsConstructor
public class QuestionServiceImpl extends ServiceImpl<QuestionMapper, Question> implements QuestionService {


    private final CommunityAdviceMapper communityAdviceMapper;

    private final ISysUserService sysUserService;

    /**
     * 添加问题
     *
     * @param questionRes 问题信息
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addQuestion(QuestionRes questionRes) {

        Long questionUserId = questionRes.getQuestionUserId();
        Long communityId = questionRes.getCommunityId();
        Long userId = SecurityUtils.getUserId();
        if (Objects.equals(questionUserId, userId)) {
            return;
        }

        // 获取租户id
        Long tenantId = questionRes.getTenantId();

        // 答复人是否是群主
        boolean isInviteUser = !Objects.equals(tenantId, questionUserId);

        if (isInviteUser) {
            return;
        }

        Question question = new Question();
        BeanUtil.copyProperties(questionRes, question);
        question.setStatus(StatusConstant.STATUS_UNREPLIED);
        baseMapper.insert(question);


        CommunityAdvice communityAdvice = new CommunityAdvice();
        communityAdvice.setTenantId(tenantId);
        communityAdvice.setCommunityId(communityId);
        communityAdvice.setUserId(questionUserId);
        communityAdvice.setTitle("您有新的问题");
        communityAdvice.setContent(questionRes.getContent());
        communityAdviceMapper.insert(communityAdvice);


    }

    @Override
    public TableDataInfo listUnReplied(QuestionPageRes questionPageRes) {

        Page<Question> page = new Page<>(questionPageRes.getPageNum(), questionPageRes.getPageSize());
        OrderItem orderItem = new OrderItem();
        if (StringUtils.isBlank(questionPageRes.getOrderByColumn())) {
            questionPageRes.setOrderByColumn("create_time");
        }
        orderItem.setColumn(questionPageRes.getOrderByColumn());
        orderItem.setAsc(Objects.equals(questionPageRes.getIsAsc(), "asc"));
        page.addOrder(orderItem);


        LambdaQueryWrapper<Question> lqw = new LambdaQueryWrapper<>();
        lqw.eq(Question::getCommunityId, questionPageRes.getCommunityId())
                .eq(Question::getStatus, StatusConstant.STATUS_UNREPLIED);
        baseMapper.selectPage(page, lqw);

        // 获取分页数据
        List<Question> questionList = page.getRecords();
        // Question数据转为QuestionVo
        List<QuestionVo> questionVoList = new ArrayList<>();
        for (Question question : questionList) {
            QuestionVo questionVo = new QuestionVo();
            BeanUtil.copyProperties(question, questionVo);
            Long questionUserId = question.getQuestionUserId();
            SysUser sysUser = sysUserService.selectUserById(questionUserId);
            questionVo.setQuestionUserName(sysUser.getUserName());
            questionVo.setQuestionUserAvatar(sysUser.getAvatar());
            questionVoList.add(questionVo);

        }

        // 封装分页信息
        TableDataInfo rspData = new TableDataInfo();
        rspData.setCode(HttpStatus.SUCCESS);
        rspData.setMsg("查询成功");
        rspData.setRows(questionVoList);
        rspData.setTotal(page.getTotal());

        return rspData;
    }

    @Override
    public QuestionVo getDetail(Long id) {

        Question question = baseMapper.selectById(id);
        if (Objects.isNull(question)) {
            return null;
        }
        QuestionVo questionVo = new QuestionVo();
        BeanUtil.copyProperties(question, questionVo);
        Long questionUserId = question.getQuestionUserId();
        SysUser sysUser = sysUserService.selectUserById(questionUserId);
        questionVo.setQuestionUserName(sysUser.getUserName());
        questionVo.setQuestionUserAvatar(sysUser.getAvatar());


        return questionVo;
    }

    /**
     * 回复问题
     *
     * @param questionReplyDto 回复信息
     */
    @Override
    public void reply(QuestionReplyDto questionReplyDto) {

        Long questionId = questionReplyDto.getQuestionId();
        LambdaQueryWrapper<Question> lqw = new LambdaQueryWrapper<>();
        lqw.eq(Question::getId, questionId)
                .eq(Question::getStatus, StatusConstant.STATUS_UNREPLIED);

        // 查询问题信息
        Question question = baseMapper.selectOne(lqw);

        if (Objects.isNull(question)) {
            return;
        }

        question.setContent(questionReplyDto.getContent());
        question.setStatus(StatusConstant.STATUS_REPLIED);
        question.setReplyTime(new Date());

        baseMapper.updateById(question);

    }


}
